/*
 * This file is part of SubLink.
 * Copyright (C) 2009  Jeferson Hultmann <hultmann@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

"use strict";

var EXPORTED_SYMBOLS = ["addCommandToDom", "removeCommandFromDom",
                        "buttonCmd", "AnimateButton",
                        "buttonClick", "buttonTooltip", "buttonCtor"];

var Cu = Components.utils;
Cu.import("resource://sublink-migreme-4.1.2.348.18-59/util.js");

var m_size = -1;
var m_imgData = null;



function addCommandToDom(doc) {
  var cmdButton = doc.createElement("command");

  cmdButton.setAttribute("id", "cmd_sublink-migreme-copy-url");
  //cmdButton.setAttribute("oncommand", "Cu.import('resource://sublink-migreme-4.1.2.348.18-59/button.js',{}).buttonCmd(event);");
  cmdButton.addEventListener("command", function(evt) {
    Cu.import('resource://sublink-migreme-4.1.2.348.18-59/button.js', null).buttonCmd(evt);
  });
  //cmdButton.setAttribute("onclick", "Cu.import('resource://sublink-migreme-4.1.2.348.18-59/button.js',{}).buttonClick(event);");
  cmdButton.addEventListener("click", function(evt) {
    Cu.import('resource://sublink-migreme-4.1.2.348.18-59/button.js', null).buttonClick(evt);
  });
  cmdButton.setAttribute("tooltiptext", "*");

  cmdButton.setAttribute("label", util.getText("button.create.label", "migre.me"));
  cmdButton.hidden = false;


  var d1 = doc.createElement("commandset");
  d1.appendChild(cmdButton);
  doc.documentElement.appendChild(d1);
}


function removeCommandFromDom(doc) {
  var cmdset = doc.getElementById("cmd_sublink-migreme-copy-url").parentNode;
  var win = cmdset.parentNode;
  win.removeChild(cmdset);
}


function buttonCtor(button) {
  var canvas = button.canvasElement;
  if (m_size !== canvas.width) {
    // new size, new image
    m_size = canvas.width;
    m_imgData = null;
  }

  var ctx = canvas.getContext("2d");

  if (m_imgData !== null) {
    ctx.putImageData(m_imgData, 0, 0);
  } else {
    ctx.clearRect(0, 0, m_size, m_size);
    loadImage(button);
  }
}


function loadImage(button) {
  var imgFav = createHtmlElement("img", button.ownerDocument);

  imgFav.onload = function(evt) {
    imgToButton(imgFav, button);
  };

  imgFav.onerror = function(evt) {
    Cu.import("resource://gre/modules/AddonManager.jsm", {})
    .AddonManager.getAddonByID("migre.me@hultmann", function(addon) {
      var imgApp = createHtmlElement("img", button.ownerDocument);
      imgApp.onload = function(evt) {
        imgToButton(imgApp, button);
      }
      imgApp.src = addon.iconURL;
    });
  };

  imgFav.src = m_size === 16 ? "http://migre.me/favicon.ico" : "http://m.migre.me/apple-touch-icon.png";
}


function imgToButton(img, button) {
  var scale = m_size / img.width;
  var ctx = button.canvasElement.getContext("2d");
  ctx.save();
  ctx.scale(scale, scale);
  ctx.drawImage(img, 0, 0);
  ctx.restore();

  m_imgData = ctx.getImageData(0, 0, m_size, m_size);
}


// nao funciona no momento (fx40), aparentemente command.onclick é ignorado
function buttonClick(evt) {
  if (evt.button !== 1 || evt.detail !== 1) {
    return;
  }

  // middle click
  var button = evt.target;
  var win = button.ownerDocument.defaultView;
  util.openPage("http://migre.me/", "tab", win);
}


function buttonCmd(evt) {
  var command = evt.target;

  var win = command.ownerDocument.defaultView;
  var browser = win.getBrowser().selectedBrowser;
  var uri = util.stringToUri(browser.currentURI.spec);//contentWindow.location.href);

  Cu.import("resource://sublink-migreme-4.1.2.348.18-59/redir-storage.js");
  var redir = redirStorage.getSavedRedir(uri);
  if (redir) {
    util.toClipboard(redir);
    AnimateButton.done(win);
    return;
  }

  function onComplete() {
    AnimateButton.waitStop(win);
  }

  function onSuccess(info) {
    AnimateButton.done(win);
    util.toClipboard(info.shortUri.spec);
  }

  function onError(errData) {
    Cu.import("resource://sublink-migreme-4.1.2.348.18-59/err.js");
    notificationError(errData, browser);
  }

  AnimateButton.waitStart(win);
  util.toClipboard("");
  Cu.import("resource://sublink-migreme-4.1.2.348.18-59/util2.js");
  convertUrl(uri, onSuccess, onError, onComplete);
}


function buttonTooltip(tooltip) {
  var win = tooltip.ownerDocument.defaultView;
  var uri = win.getBrowser().selectedBrowser.currentURI;//contentDocument.documentURIObject;
  var text;

  Cu.import("resource://sublink-migreme-4.1.2.348.18-59/redir-storage.js");
  var redir = redirStorage.getSavedRedir(uri);
  if (redir) {
    text = util.getText("button.create.saved.tooltip", redir);
  } else {
    text = util.getText("button.create.new.tooltip", "migre.me");
  }
  tooltip.setAttribute("label", text);
}


function createHtmlElement(tagName, xulDoc) {
  return xulDoc.createElementNS("http://www.w3.org/1999/xhtml", tagName);
}


var AnimateButton = {
  _getButton: function(win) {
    return win.document.getElementById("sublink-migreme-button");
  },

  waitStart: function(win) {
    var button = this._getButton(win);
    if (button.anim) {
      button.anim.finish();
    }
    button.anim = new WorkingButton(button);
    button.anim.wait();
  },

  waitStop: function(win) {
    var button = this._getButton(win);
    if (button.anim) {
      button.anim.finish();
    }
  },

  done: function(win) {
    var button = this._getButton(win);
    if (!button.anim) {
      button.anim = new WorkingButton(button);
    }
    button.anim.success();
  },

  remove: function(win) {
    delete this._getButton(win).anim;
  },

  easing: {
    success: function(d) {
      return (1 - Math.cos(Math.PI * d)) / 2;
    },
    waiting: function(d) {
      return -d;
    }
  }
};


function WorkingButton(button) {
  var win = button.ownerDocument.defaultView;
  this._win = Cu.getWeakReference(win);
  this._targetContext = button.canvasElement.getContext("2d");

  var cnv = createHtmlElement("canvas", win.document);
  cnv.width = m_size;
  cnv.height = m_size;
  cnv.getContext("2d").putImageData(m_imgData, 0, 0);
  this._canvasOriginal = cnv;
}


WorkingButton.prototype = {
  _canvasOriginal: null,
  _targetContext: null,
  _win: null,

  finish: function() {
    var win = this._win.get(); // BUG null ao clicar durante animacao
    AnimateButton.remove(win);
    if (win !== null) {
      this._renderFrame(0, "waiting"); // reset icon
    }
    this._win = null;
    this._canvasOriginal = null;
    this._targetContext = null;
  },

  wait: function() {
    this._isSuccess = false;
    this._start = this._win.get().performance.now();
    this._waitCore(this._start);
  },


  success: function() {
    this._isSuccess = true;
    this._start = this._win.get().performance.now();
    this._successCore(this._start);
  },


  _waitCore: function(now) {
    if (this._isSuccess || (this._canvasOriginal === null)) {
      return; // this is a remaining requestAnimationFrame
    }
    var duration = 9000;
    var rotation = (now - this._start) / duration;
    if (rotation > 1) {
      rotation -= 1;
    }
    this._renderFrame(rotation, "waiting");
    var me = this;
    this._win.get().requestAnimationFrame(function(timestamp) {
      me._waitCore.call(me, timestamp);
    });
  },

  _successCore: function(now) {
    var duration = 400;
    var rotation = (now - this._start) / duration;
    if (rotation > 1) {
      this.finish();
      return;
    }

    this._renderFrame(rotation, "success");
    var me = this;
    this._win.get().requestAnimationFrame(function(timestamp) {
      me._successCore.call(me, timestamp);
    });
  },


  _renderFrame: function(rotation, easeFunction) {
    var offsetY = 1;
    var size2 = m_size / 2;

    var ctx = this._targetContext;
    ctx.save();
    ctx.clearRect(0, 0, m_size, m_size);
    ctx.translate(size2, size2 + offsetY);
    var rotation2 = AnimateButton.easing[easeFunction](rotation);
    ctx.rotate(2 * Math.PI * rotation2);
    ctx.drawImage(this._canvasOriginal, -size2, -size2 - offsetY);
    ctx.restore();
  }
};
